#include "archprofilewizard.h"
#include "netutils.h"
#include <QtGui>
//#include <QtCore>
//#include <QWizard>
//

QMap<QString, QString> profileMap;

ArchProfileWizard::ArchProfileWizard(QWidget *parent) : QWizard(parent)
{
	setWindowTitle(tr("Arch Assistant Instant Connection Wizard"));
	setWindowIcon(QIcon(":/archlogo.png"));
	//setMinimumSize(800,800);
	
	setPage(Page_Device_Selector, new IntroPage);
	setPage(Page_Wifi_Selector, new WifiSelectorPage(this));
	setPage(Page_Wifi_Parameters, new WifiParametersPage);
	setPage(Page_Connection, new ConnectionPage(this));
	setStartId(Page_Device_Selector);
	
	setOption(QWizard::NoCancelButton, true);
	setOption(QWizard::HaveCustomButton1, true);
	setButtonText(QWizard::CustomButton1, tr("&Cancel"));
	setButtonText(QWizard::BackButton, QString("< %1").arg( tr("&Back") ));
	setButtonText(QWizard::NextButton, QString("%1 >").arg( tr("&Next") ));
	setButtonText(QWizard::FinishButton, tr("&Finish"));
	
	QList<QWizard::WizardButton> layout;
	layout << QWizard::CustomButton1 << QWizard::Stretch << QWizard::BackButton << QWizard::NextButton << QWizard::FinishButton ;
	setButtonLayout(layout);	
	
	//setButton(QWizard::CancelButton, QWizard::CustomButton1);
	
	connect( button(QWizard::CustomButton1), SIGNAL( clicked() ), this, SLOT( dontLeaveNow() ) );
	/*
	connect( button(QWizard::CancelButton), SIGNAL( clicked() ), this, SLOT( dontLeaveNow() ) );
	connect( this, SIGNAL( rejected() ), this, SLOT( dontLeaveNow() ) );
	connect( this, SIGNAL( accepted() ), this, SLOT( dontLeaveNow() ) );
	connect( this, SIGNAL( close() ), this, SLOT( dontLeaveNow() ) );
	connect( this, SIGNAL( destroyed() ), this, SLOT( dontLeaveNow() ) );
	*/
}

int ArchProfileWizard::dontLeaveMessageBox()
{
	return QMessageBox::question(
					this,
					tr("Arch Assistant Connection Wizard"),
					tr("Do you want to quit the wizard ?"),
					tr("&Resume"), tr("&Leave wizard"),
					0, 1);
}

void ArchProfileWizard::dontLeaveNow()
{
	switch ( dontLeaveMessageBox() )
			{
				case 1:
					reject();
			}
}

void ArchProfileWizard::closeEvent(QCloseEvent *event)
{
	switch ( dontLeaveMessageBox() )
			{
				case 0:
					event->ignore();
					break;
				case 1:
					event->accept();
					break;
				//default:
				//	event->ignore();
			}
}

/* *** INTRO page *** */

IntroPage::IntroPage(QWidget *parent) : QWizardPage(parent)
{
	setTitle(tr("Select a Network Interface"));
	setSubTitle(tr("Choose an ethernet or wireless interface to connect to a network near you."));
	setPixmap(QWizard::LogoPixmap, QPixmap(":/wizard64.png"));
	
	labelVendor = new QLabel();
	labelVendor->setWordWrap(true);
	labelVendor->setAlignment(Qt::AlignJustify);
	
	QStringList interfacesList = netUtils->getInterfacesList();
	cbInterfaceName = new QComboBox;
	connect( cbInterfaceName, SIGNAL( currentIndexChanged(const QString&) ), this, SLOT( cbInterfaceNameStateChanged(const QString&) ) );
	
	for (int i = 0; i < interfacesList.size(); ++i)
	{
		if ( netUtils->getInterfaceWifiStatus(interfacesList.at(i)) == true )
		{
			cbInterfaceName->addItem(
				QIcon(":/wifi_radar_22x22.png"),
				QString("%1 (%2)").arg( interfacesList.at(i) ).arg( tr("WIFI") ),
				QVariant("wifi") );
		}
		else
		{
			cbInterfaceName->addItem(
				QIcon(":/theme2_none.png"),
				QString("%1 (%2)").arg( interfacesList.at(i) ).arg( tr("ETHERNET") ),
				QVariant("ethernet") );
		}
	}
	
	QHBoxLayout *hlay1 = new QHBoxLayout;
	hlay1->addStretch();
	//hlay1->addWidget(labelInterface);
	hlay1->addWidget(cbInterfaceName);
	//hlay1->addWidget(pbQuit);
	hlay1->addStretch();
	
	QHBoxLayout *hlay2 = new QHBoxLayout;
	//hlay2->addStretch();
	hlay2->addWidget(labelVendor);
	//hlay2->addWidget(cbInterfaceName);
	//hlay1->addWidget(pbQuit);
	//hlay2->addStretch();
	
	QGroupBox *groupBox1 = new QGroupBox(tr("Network interface name and type"));
	groupBox1->setLayout(hlay1);
	
	QGroupBox *groupBox2 = new QGroupBox(tr("Interface description"));
	groupBox2->setLayout(hlay2);
	
	QVBoxLayout *layout = new QVBoxLayout;
	layout->addSpacing(25);
	layout->addWidget(groupBox1);//layout->addLayout(hlay1);//layout->addWidget(cbInterfaceName);
	layout->addSpacing(25);
	layout->addWidget(groupBox2);
	//layout->addWidget(label1);
	//layout->addWidget(labelVendor);
	
	setLayout(layout);
}

int IntroPage::nextId() const
{
	QVariant deviceType( cbInterfaceName->itemData( cbInterfaceName->currentIndex() ) );
	
	if (deviceType.toString() == "ethernet") //if ( cbInterfaceName->currentText().contains("ETHERNET") )
		return ArchProfileWizard::Page_Connection;
	else
		return ArchProfileWizard::Page_Wifi_Selector;
}

void IntroPage::cbInterfaceNameStateChanged(const QString &text)
{
	// Display tooltip with network card manufacturer name. 
	QStringList vendorName = text.split(" ");
	if (vendorName.size() > 0)
	{
		cbInterfaceName->setToolTip( netUtils->getVendorNameForInterface(vendorName.at(0)) );
		labelVendor->setText( QString("<span style='color: blue;'><em>%1</em></span>" ).arg( netUtils->getVendorNameForInterface(vendorName.at(0)) ) );
		
		if ( netUtils->getInterfaceWifiStatus(vendorName.at(0)) == true )
			profileMap["type"] = "wireless";
		else
			profileMap["type"] = "ethernet";
			
		profileMap["interface"] = vendorName.at(0);
	}
	else
	{
		cbInterfaceName->setToolTip("");
		labelVendor->setText( QString("<em>%1</em>" ).arg( tr("Unknown") ) );
	}
}

/* *** Wifi Selector page *** */

WifiSelectorPage::WifiSelectorPage(QWidget *parent) : QWizardPage(parent)
{
	setTitle(tr("Select a Wireless Network"));
	setSubTitle(tr("Choose an a wireless network in the list below."));
	setPixmap(QWizard::LogoPixmap, QPixmap(":/wizard64.png"));
	
	netUtils = new NetUtils();
	
	busyString = tr("<em>Please wait while remote networks are being scanned ...</em>");
	pageComplete = false;
	shot = 1;
	
	busyTimer = new QTimer(this);
	iwconfigProcess = new QProcess();
	
	statusLabel = new QLabel("");
	
	QStringList headers;
	headers << trUtf8("Essid") << trUtf8("Signal") << trUtf8("Channel") << trUtf8("Mode") << trUtf8("Protocol");
	
	treeWidget = new QTreeWidget(this);
	treeWidget->setColumnCount(5);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	treeWidget->setSelectionMode(QAbstractItemView::SingleSelection);
	treeWidget->setHeaderLabels(headers);
	
	pbRescan = new QPushButton(QIcon(":/reload.png"), tr("&Reload"));
	
	leFakeEssid = new QLineEdit;
	leFakeEssid->setVisible(false);
	leFakeProtocol = new QLineEdit;
	leFakeProtocol->setVisible(false);
	
	QHBoxLayout *hlay1 = new QHBoxLayout;
	hlay1->addStretch();
	hlay1->addWidget(statusLabel);
	hlay1->addWidget(pbRescan);
	//hlay1->addWidget(pbQuit);
	//hlay1->addStretch();
	
	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(treeWidget);
	layout->addWidget(leFakeEssid);
	layout->addLayout(hlay1);//layout->addWidget(pbRescan);
	layout->addWidget(leFakeProtocol);
	setLayout(layout);
	
	//connect( wizard()->button(QWizard::BackButton), SIGNAL( clicked() ), this, SLOT( nextId() ) ); 
	connect( pbRescan, SIGNAL( clicked() ), this, SLOT( scanNetwork() ) );
	connect( iwconfigProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(scanProcessEnded(int, QProcess::ExitStatus)));
	connect( treeWidget, SIGNAL( itemClicked ( QTreeWidgetItem*, int ) ), this, SLOT( networkItemSelected() ) );
	connect( treeWidget, SIGNAL( itemDoubleClicked ( QTreeWidgetItem*, int ) ), parent, SLOT( next() ) );
	connect( busyTimer, SIGNAL( timeout() ), this, SLOT( timerTimeout() ) );
	
	registerField("fake.essid", leFakeEssid);
	registerField("fake.protocol", leFakeProtocol);
}

void WifiSelectorPage::initializePage()
{
	//wizard()->button(QWizard::BackButton)->setDisabled(true);
	if ( iwconfigProcess->state() == QProcess::NotRunning )
	{
		scanNetwork();
	}
}

int WifiSelectorPage::nextId() const
{
	return ArchProfileWizard::Page_Wifi_Parameters;
}

bool WifiSelectorPage::isComplete() const
{
	return pageComplete;
}

void WifiSelectorPage::timerTimeout()
{
	if (shot == 1)
	{
		statusLabel->setText("");
		shot = 0;
	}
	else
	{
		statusLabel->setText(busyString);	
		shot = 1;
	}
}

void WifiSelectorPage::networkItemSelected()
{
	pageComplete = true;
	leFakeEssid->setText(treeWidget->currentItem()->text(0));
	leFakeProtocol->setText(treeWidget->currentItem()->text(4));
	profileMap["mac_address"] = treeWidget->currentItem()->data(0, Qt::ToolTipRole).toString();
	emit QWizardPage::completeChanged();
}

void WifiSelectorPage::scanNetwork()
{
	// check if interface is down before launching iwlist scan
	if (netUtils->getInterfaceStatus(netUtils->getWifiInterfaceName()) == "down")
	{
		QProcess *ifconfigUpProcess = new QProcess();
		ifconfigUpProcess->start(netUtils->ifconfigBinary + " " + netUtils->getWifiInterfaceName() + " up");
	
		if (!ifconfigUpProcess->waitForStarted())
		{
			qDebug("error ifconfigUpProcess");
			return;
		}
		if (!ifconfigUpProcess->waitForFinished())
		{
			qDebug("error ifconfigUpProcess");
			return;
		}
	}
	
	// Launch iwlist scan process
	iwconfigProcess->setProcessChannelMode(QProcess::MergedChannels);
	iwconfigProcess->start( netUtils->iwlistBinary + " " + netUtils->getWifiInterfaceName() + " scan" );
	
	if (!iwconfigProcess->waitForStarted())
	{
		qDebug("error iwlist scan");
		return ;
	}
	
	pageComplete = false;
	emit QWizardPage::completeChanged();
	setCursor(Qt::WaitCursor);
	treeWidget->setDisabled(true);
	pbRescan->setDisabled(true);
	busyTimer->start(1000);
	statusLabel->setText(busyString);
}

void WifiSelectorPage::scanProcessEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	QString str;
	
	switch (exitStatus)
	{
		case QProcess::NormalExit:
			if (exitCode == 0)
			{
				qDebug() << "process is a success ! ExitCode = " << qPrintable(str.setNum(exitCode)) << endl;
			}
			else
			{
				QMessageBox::warning(
									this,
									tr("Arch Assistant"),
			 						tr("Error : process ended with errors."),
									QMessageBox::Ok
									);
				qDebug() << "process returned an error ! ExitCode = " << qPrintable(str.setNum(exitCode)) << endl;
			}
			break;
		case QProcess::CrashExit:
			QMessageBox::warning(
								this,
		 						tr("Arch Assistant"),
								tr("Error : process crashed."),
								QMessageBox::Ok
								);
			qDebug() << "process crashed ! " << endl;

			break;
		default:
			break;
	}
	
	busyTimer->stop();
	statusLabel->setText("");
	treeWidget->setDisabled(false);
	pbRescan->setDisabled(false);
	setCursor(Qt::ArrowCursor);
	
	updateTreeWidgetItems();
}

void WifiSelectorPage::updateTreeWidgetItems()
{
	QTextStream stream(iwconfigProcess->readAll().data());
	QString line;
	QString buffer = "";
	int cellCount = 0;
	QMap<int, QStringList> cellsMap;

	do {
		
		line = stream.readLine().trimmed();

		if ( line.startsWith("Cell") )
		{
			if (cellCount > 0)
				buffer = buffer + "<<#>>"; // cell change

			cellCount ++;
		}

		if (line.size() > 0)
			buffer = buffer + line + "<<>>"; // line change
	
	} while (!line.isNull());

	cellsMap = netUtils->parseIwlistScan(buffer);
	
	treeWidget->clear();
	
	for (int i=0 ; i < cellsMap.size() ; i++)
	{
		QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
		itm->setText(0, cellsMap[i].at(0));
		itm->setText(1, cellsMap[i].at(2));
		itm->setText(2, cellsMap[i].at(1));
		itm->setText(3, cellsMap[i].at(4));
		itm->setText(4, cellsMap[i].at(5));
		if (cellsMap[i].at(3) == "on")
			itm->setIcon(0, QIcon(":/lock.png"));
		itm->setData (0, Qt::ToolTipRole, QVariant(cellsMap[i].at(6)));
// 		qDebug() << qPrintable(cellsMap[i].at(0)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(1)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(2)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(3)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(4)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(5)) << endl;
// 		qDebug() << "////" << endl;
	}
	
	treeWidget->setSortingEnabled(true);
	treeWidget->sortByColumn(1, Qt::DescendingOrder);
	treeWidget->resizeColumnToContents ( 0 );
	treeWidget->resizeColumnToContents ( 1 );
	treeWidget->resizeColumnToContents ( 2 );
	treeWidget->resizeColumnToContents ( 3 );
	treeWidget->resizeColumnToContents ( 4 );
}

/* *** Wifi Parameters page *** */

WifiParametersPage::WifiParametersPage(QWidget *parent) : QWizardPage(parent)
{
	setTitle(tr("Specify Connection Parameters"));
	setSubTitle(tr("Please provide the password and adjust other settings in order to connect to the selected wireless network."));
	setPixmap(QWizard::LogoPixmap, QPixmap(":/wizard64.png"));

	QLabel *labelProtocols = new QLabel(tr("Security protocol : "));
	QLabel *labelEssid = new QLabel(tr("Network name (SSID) : "));
	QLabel *labelPwd = new QLabel(tr("Password : "));
	
	QStringList protocolsList;
	protocolsList << "WEP" << "WPA" << "None";
	
	cbProtocols = new QComboBox;
	cbProtocols->addItems(protocolsList);
	connect( cbProtocols, SIGNAL( currentIndexChanged(const QString&) ), this, SLOT( cbProtocolsStateChanged(const QString&) ) );
	
	leEssid = new QLineEdit;
	lePwd = new QLineEdit;
	/*
	QHBoxLayout *hlay1 = new QHBoxLayout;
	hlay1->addWidget(labelProtocols);
	hlay1->addStretch();
	hlay1->addWidget(cbProtocols);
	hlay1->addStretch();
	
	QHBoxLayout *hlay2 = new QHBoxLayout;
	hlay2->addWidget(labelEssid);
	hlay2->addStretch();
	hlay2->addWidget(leEssid);
	hlay2->addStretch();
	
	QHBoxLayout *hlay3 = new QHBoxLayout;
	hlay3->addWidget(labelPwd);
	hlay3->addStretch();
	hlay3->addWidget(lePwd);
	hlay3->addStretch();
	*/
	
	QGridLayout *glay = new QGridLayout;
	glay->addWidget(labelProtocols,0,0);
	glay->addWidget(cbProtocols,0,1);
	glay->addWidget(labelEssid,1,0);
	glay->addWidget(leEssid,1,1);
	glay->addWidget(labelPwd,2,0);
	glay->addWidget(lePwd,2,1);
	
	QGroupBox *groupBox = new QGroupBox(tr("Connection settings"));
	groupBox->setLayout(glay);
	
	QVBoxLayout *layout = new QVBoxLayout;
	//layout->addSpacing(25);
	layout->addStretch();
	layout->addWidget(groupBox);//layout->addLayout(glay);
	layout->addStretch();
	/*
	//layout->addWidget(labelProtocols);
	//layout->addWidget(cbProtocols);
	layout->addLayout(hlay1);
	layout->addSpacing(25);
	layout->addLayout(hlay2);
	//layout->addWidget(labelEssid);
	//layout->addWidget(leEssid);
	layout->addSpacing(25);
	layout->addLayout(hlay3);
	//layout->addWidget(labelPwd);
	//layout->addWidget(lePwd);
	*/
	setLayout(layout);
	
	lePwd->setFocus();
	
	registerField("cbProtocols", cbProtocols);
	registerField("leEssid*", leEssid);
	registerField("lePwd*", lePwd);
}

void WifiParametersPage::initializePage()
{
	QString protocolStr = field("fake.protocol").toString();
	
	if (protocolStr == "WEP")
		cbProtocols->setCurrentIndex(0);
	else if (protocolStr == "WPA")
		cbProtocols->setCurrentIndex(1);
	else
		cbProtocols->setCurrentIndex(2);
	
	leEssid->setText( field("fake.essid").toString() );
	lePwd->setFocus();
	profileMap["protocol"] = cbProtocols->currentText();
}

int WifiParametersPage::nextId() const
{
	return ArchProfileWizard::Page_Connection;
}

void WifiParametersPage::cbProtocolsStateChanged(const QString &text)
{
	profileMap["protocol"] = text;//cbProtocols->currentText();
}

/* *** Connection page *** */

ConnectionPage::ConnectionPage(QWidget *parent) : QWizardPage(parent)
{
	setTitle(tr("Apply Network Configuration"));
	setSubTitle(tr("Click the button below to initiate the connection."));
	setPixmap(QWizard::LogoPixmap, QPixmap(":/wizard64.png"));
	setFinalPage(true);
	
	wizardIsComplete = false;
	pageComplete = false;
	
	netUtils = new NetUtils();
	netcfgProcess = new QProcess;

	leProfileName = new QLineEdit;
	
	pbConnect = new QPushButton(QIcon(":/wizard.png"), tr("CONNECT NOW"));
	QFont font = pbConnect->font();
	font.setBold(true);
	pbConnect->setFont(font);
	
	chStopProfiles = new QCheckBox( tr("Deactivate active profiles") );
	chStopProfiles->setChecked(true);
	chSaveProfile = new QCheckBox( QString("%1 : ").arg( tr("Save this profile") ) );
	chSaveProfile->setChecked(false);
	chSaveProfile->setDisabled(true);
	
	sumup1 = "";
	sumup2 = "";
	labelSummary = new QLabel( QString("<em>%1<br>%2</em>").arg(sumup1).arg(sumup2) );
	labelSummary->setWordWrap(true);
	labelSummary->setAlignment(Qt::AlignHCenter);
	
	QHBoxLayout *hlay0 = new QHBoxLayout;
	hlay0->addWidget(labelSummary);

	QHBoxLayout *hlay1 = new QHBoxLayout;
	hlay1->addStretch();
	hlay1->addWidget(chSaveProfile, Qt::AlignLeft);
	hlay1->addWidget(leProfileName, Qt::AlignLeft);
	hlay1->addStretch();
	
	QHBoxLayout *hlay2 = new QHBoxLayout;
	hlay2->addStretch();
	hlay2->addWidget(pbConnect);
	hlay2->addStretch();
	
	
	QVBoxLayout *vlay3 = new QVBoxLayout;
	vlay3->addLayout(hlay1, Qt::AlignLeft);
	vlay3->addWidget(chStopProfiles, Qt::AlignLeft);
	vlay3->addStretch();
	
	QGroupBox *groupBox0 = new QGroupBox(tr("Connection configuration summary"));
	groupBox0->setLayout(hlay0);
	
	QGroupBox *groupBox1 = new QGroupBox(tr("Miscellaneous options"));
	groupBox1->setLayout(vlay3);//groupBox1->setLayout(hlay1);
	
	QGroupBox *groupBox2 = new QGroupBox(tr("Apply network configuration"));
	groupBox2->setLayout(hlay2);
	
	QVBoxLayout *layout = new QVBoxLayout;
	layout->addStretch();
	layout->addWidget(groupBox0);
	layout->addWidget(groupBox1);
	layout->addWidget(groupBox2);
	layout->addStretch();

	setLayout(layout);
	
	connect( this, SIGNAL( closeWizard() ), parent, SLOT( accept() ) );
	connect( pbConnect, SIGNAL( clicked() ), this, SLOT( connectionRequested() ) );
	connect( chSaveProfile, SIGNAL( stateChanged(int) ), this, SLOT( editableProfileName() ) );
	//connect( netcfgProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(netcfgProcessEnded(int, QProcess::ExitStatus)));
}

void ConnectionPage::initializePage()
{
	sumup1 = QString("Interface : %1&nbsp;/&nbsp;Connection type : %2")
							.arg(profileMap["interface"])
							.arg(profileMap["type"]);
							
	if ( profileMap["type"] == "wireless" )
	{
		chSaveProfile->setDisabled(false);
		chSaveProfile->setChecked(false);
		leProfileName->setDisabled(true);
		leProfileName->setText(generateProfileName());
		sumup2 = QString("Essid : '%1'&nbsp;/&nbsp;Security protocol : %2")
							.arg(field("leEssid").toString())
							.arg(profileMap["protocol"]);
	}
	else
	{
		chSaveProfile->setDisabled(true);
		chSaveProfile->setChecked(false);
		leProfileName->setDisabled(true);
		leProfileName->setText("last");
		sumup2 = "";
	}
		
	labelSummary->setText( QString("<span style='color: blue;'><em>%1<br>%2</em></span>").arg(sumup1).arg(sumup2));
	
	pbConnect->setEnabled(true);
	
	wizard()->button(QWizard::CustomButton1)->setDisabled(false);
}

int ConnectionPage::nextId() const
{
	return -1;
}

bool ConnectionPage::isComplete() const
{
	return pageComplete;
}

void ConnectionPage::editableProfileName()
{
	if (chSaveProfile->isChecked())
		leProfileName->setDisabled(false);
	else
		leProfileName->setDisabled(true);
}

void ConnectionPage::connectionRequested()
{
	pbConnect->setEnabled(false);
	
	//ConnectionDlg *connDlg = new ConnectionDlg(this, "ping www.clubic.com -c 6");
	//connDlg->exec();
	//return;
	
	const QString profileName = "last";
	
	if ( generateProfile(profileName) == true )
	{
		//setCursor(Qt::WaitCursor);
		
		//QString whichProcess = "netcfg2 -i " + profileMap["interface"];
		QString whichProcess = netUtils->netcfgBinary + " -i " + profileMap["interface"];
		if ( chStopProfiles->isChecked() )
			whichProcess = netUtils->netcfgBinary + " -a";//whichProcess = "netcfg2 -a";
		
		QProcess *preProcess = new QProcess();
		preProcess->start(whichProcess);	
		if (!preProcess->waitForStarted())
		{
			qDebug("error ifconfigUpProcess");
			return;
		}
		if (!preProcess->waitForFinished())
		{
			qDebug("error ifconfigUpProcess");
			return;
		}
	
		ConnectionDlg *connDlg = new ConnectionDlg(this, netUtils->netcfgBinary + " " + profileName);
		connDlg->exec();
		//return;
		
		//netcfgProcess->start("netcfg2 " + profileName);
		
		//wizard()->button(QWizard::CustomButton1)->setDisabled(true);
		pageComplete = true;
		emit QWizardPage::completeChanged();
		
		if (wizardIsComplete)
			emit closeWizard();
	}
}

const QString ConnectionPage::generateProfileName()
{
	return field("leEssid").toString().replace(" ", "_").toLower() + "_" + profileMap["mac_address"];
}

bool ConnectionPage::generateProfile(const QString & profileName)
{
	QString filePath = netUtils->profilesDirName + "/" + profileName;

	QFile file(filePath);
	if (!file.open(QFile::WriteOnly | QFile::Text))
	{
		QMessageBox::warning(
				this,
				tr("Application"),
				tr("Cannot write file '%1'.").arg(filePath)
				);
		return false;
	}
	
	QStringList fileBits;
	
	fileBits << "CONNECTION=\"" + profileMap["type"] + "\"";
	fileBits << "DESCRIPTION=\"Profile generated by arch assistant instant connection wizard\"";
	fileBits << "INTERFACE=\"" + profileMap["interface"] + "\"";
	fileBits << "HOSTNAME=\"" + netUtils->getHostname() + "\"";
	fileBits << "IP=\"dhcp\"";
	fileBits << "IFOPTS=\"dhcp\"";
	
	if ( profileMap["type"] == "wireless" )
	{
		fileBits << "# Wireless Settings";
		fileBits << "SECURITY=\"" + profileMap["protocol"].toLower() + "\"";
		fileBits << "ESSID=\"" + field("leEssid").toString() + "\"";
		fileBits << "KEY=\"" + field("lePwd").toString() + "\"";
		fileBits << "SCAN=\"no\"";
		if ( !profileMap["mac_address"].isEmpty() )
			fileBits << "MAC_ADDRESS=\"" + profileMap["mac_address"].toLower() + "\"";
	}
	
	// fill text file
	QTextStream in(&file);
	
	for (int i=0 ; i < fileBits.count() ; ++i)
		in << fileBits.at(i) << endl;//qDebug() << qPrintable(fileBits.at(i)) << endl;

	file.close();
	
	return true;
}

void ConnectionPage::myslot()
{
	qDebug("connection is Ok, wizard is completed");
	
	wizardIsComplete = true;
	
	if ( chSaveProfile->isEnabled() )
	{
		if ( chSaveProfile->isChecked() )
			saveProfile();
	}
}

void ConnectionPage::saveProfile()
{
	QString targetProfileName = netUtils->checkProfileNameSyntax(leProfileName->text());//generateProfileName();
	
	QString sourceFilename = QString("%1/%2")
								.arg(netUtils->profilesDirName)
								.arg("last");
	QString targetFilename = QString("%1/%2")
								.arg(netUtils->profilesDirName)
								.arg(netUtils->checkProfileNameSyntax(targetProfileName));//.arg(field("leEssid").toString().replace(" ", "_").toLower() + "_" + profileMap["mac_address"]);
	
	if (QFile::exists(targetFilename))
		QFile::remove(targetFilename);
		
	if (!QFile::copy(sourceFilename, targetFilename))
	{
		QMessageBox::warning(
							this,
		  					tr("Arch Assistant"),
			 				tr("Error : could not write profile file."),
							QMessageBox::Ok
							);
	}
}

/*
void ConnectionPage::netcfgProcessEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	QString str;
	
	switch (exitStatus)
	{
		case QProcess::NormalExit:
			if (exitCode == 0)
			{
				pageComplete = true;
				emit QWizardPage::completeChanged();
				qDebug() << "process is a success ! ExitCode = " << qPrintable(str.setNum(exitCode)) << endl;
			}
			else
			{
				QMessageBox::warning(
									 this,
		  tr("Arch Assistant"),
			 tr("Error : process ended with errors."),
				QMessageBox::Ok
									);
				qDebug() << "process returned an error ! ExitCode = " << qPrintable(str.setNum(exitCode)) << endl;
			}
			break;
		case QProcess::CrashExit:
			QMessageBox::warning(
								 this,
		 tr("Arch Assistant"),
			tr("Error : process crashed."),
			   QMessageBox::Ok
								);
			qDebug() << "process crashed ! " << endl;

			break;
		default:
			break;
	}
	
	setCursor(Qt::ArrowCursor);
}
*/


/* *** Connection Dialog *** */

ConnectionDlg::ConnectionDlg(QWidget *parent, const QString command) : QDialog(parent)
{
	setWindowTitle(tr("Arch Assistant"));
	setWindowIcon(QIcon(":/archlogo.png"));
		connect( this, SIGNAL( signalConnectionIsOk() ), parent, SLOT( myslot() ) );
	
	netUtils = new NetUtils();
	
	shot = 1;
	
	busyString = tr("<em>please wait ...</em>");
	busyTimer = new QTimer(this);
	connect( busyTimer, SIGNAL( timeout() ), this, SLOT( timerTimeout() ) );
	
	netcfgProcess = new QProcess;
	netcfgProcess->setProcessChannelMode(QProcess::ForwardedChannels);
	connect( netcfgProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(netcfgProcessEnded(int, QProcess::ExitStatus)));
	connect( netcfgProcess, SIGNAL(readyReadStandardError()),this, SLOT(netcfgReadError()));
	
	QLabel *titleLabel = new QLabel(
					QString("<p style='border: medium double rgb(250,0,255);'><table width='100%' cellpadding='4' cellspacing='4' align='center' style='border-style: solid; border-color: black; border-width: 1px; background-color: white;'><tr><td align='right'><img src=':/archlogo.png'></img></td><td align='left' valign='middle' style='color: blue;'><b>%1</b></td></tr></table></p>")
					.arg( tr("Connecting Network") )
					);
	
	mainLabel = new QLabel(busyString);
	mainLabel->setAlignment(Qt::AlignHCenter);
	mainLabel->setWordWrap(true);
	
	pbAbort = new QPushButton(tr("Abort"));
	connect( pbAbort, SIGNAL( clicked() ), this, SLOT( dontLeaveNow() ) );
	
	pbQuit = new QPushButton(tr("Ok"));
	connect( pbQuit, SIGNAL( clicked() ), this, SLOT( reject() ) );
	
	QHBoxLayout *hlay1 = new QHBoxLayout;
	hlay1->addStretch();
	hlay1->addWidget(pbAbort);
	hlay1->addWidget(pbQuit);
	//hlay1->addStretch();
	
	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(titleLabel, Qt::AlignHCenter);
	layout->addSpacing(25);
	layout->addWidget(mainLabel, Qt::AlignHCenter);
	layout->addSpacing(25);
	layout->addLayout(hlay1);//layout->addWidget(pbAbort);
	setLayout(layout);

	resize(300,125);
	
	if ( !command.isEmpty() )
	{
		netcfgProcess->start(command);//netcfgProcess->start("ping www.clubic.com -c 6");
		busyTimer->start(1000);
		setCursor(Qt::WaitCursor);
		pbQuit->setDisabled(true);
	}
}

void ConnectionDlg::netcfgReadError()
{
	QString errmsg = netcfgProcess->readAllStandardError();
	qDebug( qPrintable( errmsg ) );
}

void ConnectionDlg::closeEvent(QCloseEvent *event)
{
	event->ignore();
}

void ConnectionDlg::dontLeaveNow()
{
	switch(
			QMessageBox::warning(
							this,
							tr("Arch Assistant Connection Wizard"),
							tr("Do you want to abort the connection process ?"),
							tr("&Resume"), tr("&Abort"),
							0, 1)
			)
	{
		case 1:
			if (netcfgProcess->state() == QProcess::Running || netcfgProcess->state() == QProcess::Starting )
				netcfgProcess->terminate();
	}
}

void ConnectionDlg::netcfgProcessEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	busyTimer->stop();	
	
	QString str;
	
	switch (exitStatus)
	{
		case QProcess::NormalExit:
			if (exitCode == 0)
			{
				mainLabel->setText( QString( "<B>%1</B>" ).arg( tr("Profile successfully connected !") ) );
				emit signalConnectionIsOk();
				qDebug() << "process is a success ! ExitCode = " << qPrintable(str.setNum(exitCode)) << endl;
			}
			else
			{
				QMessageBox::warning(
								this,
								tr("Arch Assistant"),
								tr("Error : process ended with errors."),
								QMessageBox::Ok
									);
				mainLabel->setText(tr("Sorry, connection process ended with errors"));
				qDebug() << "process returned an error ! ExitCode = " << qPrintable(str.setNum(exitCode)) << endl;
			}
			break;
		
		case QProcess::CrashExit:
			QMessageBox::warning(
								this,
								tr("Arch Assistant"),
								tr("Error : process crashed."),
								QMessageBox::Ok
								);
			mainLabel->setText(tr("Sorry, connection process crashed"));
			qDebug() << "process crashed ! " << endl;
			break;
			
		default:
			break;
	}
	
	setCursor(Qt::ArrowCursor);
	pbAbort->setDisabled(true);
	pbQuit->setDisabled(false);
	
	//emit ConnectionPage::myslot();
}

void ConnectionDlg::timerTimeout()
{
	// blinking magic !
	if (shot == 1)
	{
		mainLabel->setText("");
		shot = 0;
	}
	else
	{
		mainLabel->setText(busyString);
		shot = 1;
	}
}
//
