//
// C++ Implementation: networkstopdialog
//
// Description: 
//
//
// Author: root <root@carrouf-G1>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "networkstopdialog.h"
#include "netutils.h"
#include <QtGui>

NetworkStopDialog::NetworkStopDialog(QWidget * parent, Qt::WFlags f, int mode, QString profileName)
	: QDialog(parent, f)
{
	netUtils = new NetUtils();
	
	profileNameRequested = profileName;
	stopProcess = new QProcess();
	
	treeWidget = new QTreeWidget(this);
	//treeWidget->setColumnCount(2);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	
	pbClose = new QPushButton(trUtf8("&Cancel"), this);
	pbClose->setIcon(QIcon(":/fileclose.png"));
	pbOk = new QPushButton(trUtf8("&Disconnect"), this);
	pbOk->setIcon(QIcon(":/cr22-action-wired.png"));
	if (profileNameRequested.size() == 0)
		pbOk->setDisabled(true);
	
	layout = new QHBoxLayout;
	layout->addStretch();
	layout->addWidget( pbOk );
	layout->addWidget( pbClose );
	//layout->addLayout( layoutV );
	
	QVBoxLayout *layoutV = new QVBoxLayout;
	//layoutV->addStretch();
	layoutV->addWidget( treeWidget );
	layoutV->addLayout( layout );
	
	QString titleTxt;
	errorsCount = 0;
	
	targetMode = mode;
	switch (targetMode)
	{
		case 0:
			titleTxt = tr("profiles");
			break;
		case 1:
			titleTxt = tr("interfaces");
			break;
	}
	
	updateTreeWidgetItems();
	
	//setSizeGripEnabled(true);
	setLayout( layoutV );
	setModal(true);
	setWindowTitle ( tr("Arch Assistant - Disconnect active %1").arg(titleTxt) );
	setWindowIcon(QIcon(":/archlogo.png"));
	setMinimumSize(500, 300);
	
	connect( pbClose, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( disconnectProfilesOrInterfaces() ) );
	connect(stopProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(jobEnded(int, QProcess::ExitStatus)));
	connect(treeWidget, SIGNAL(itemChanged(QTreeWidgetItem *, int)),this, SLOT(treeWidgetPressed()));
	//connect(treeWidget, SIGNAL(itemChanged(QTreeWidgetItem *, int)),this, SLOT(treeWidgetPressed(QTreeWidgetItem *,int)));
	//connect(stopProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(networkProfileProcessOutput()));
}

void NetworkStopDialog::treeWidgetPressed()//void NetworkStopDialog::treeWidgetPressed(QTreeWidgetItem * item, int column)
{
	int itemsCheckedCount = 0;
	
	QTreeWidgetItemIterator joe( treeWidget , QTreeWidgetItemIterator::Checked );
	while (*joe)
	{
		itemsCheckedCount ++;
		++joe;
	}
	//qDebug( qPrintable( QString::number(itemsCheckedCount) ) );
	if (itemsCheckedCount > 0)
		pbOk->setDisabled(false);
	else
		pbOk->setDisabled(true);
}

void NetworkStopDialog::updateTreeWidgetItems()
{
	//NetUtils *netUtils = new NetUtils();
	QStringList labelsList;
	//QStringList list;
	QStringList entryList;
	entryList.clear();
	treeWidget->clear();
	int i;
	int entryCount = 0;
	
	if (targetMode == 0)
	{
		entryList = netUtils->getActiveProfilesList();
		labelsList << tr("Profile Name");
		treeWidget->setColumnCount(labelsList.count());
		treeWidget->setHeaderLabels(labelsList);

		for (i = 0; i < entryList.size(); ++i)
		{
			QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
			//itm->setFlags(Qt::ItemIsEditable | Qt::ItemIsUserCheckable);
			if ( profileNameRequested.size() > 0 && profileNameRequested == entryList.at(i) )
				itm->setCheckState(0, Qt::Checked);
			else
				itm->setCheckState(0, Qt::Unchecked);
			itm->setText(0, entryList.at(i));
			if (netUtils->getConnectionForProfile(entryList.at(i)).toLower() == "ethernet")
				itm->setIcon(0, QIcon(":/theme2_none.png"));
			else
				itm->setIcon(0, QIcon(":/wifi_radar_22x22.png"));
			entryCount ++;
		}
	}
	else if (targetMode == 1)
	{
		//QString wlanDeviceName = netUtils->getWifiInterfaceName();
		entryList = netUtils->getInterfacesList();
		labelsList << tr("Interface Name") << tr("Interface Type");
		treeWidget->setColumnCount(labelsList.count());
		treeWidget->setHeaderLabels(labelsList);

		for (i = 0; i < entryList.size(); ++i)
		{
			if (netUtils->getInterfaceStatus(entryList.at(i)) == "up")
			{
				QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
				//itm->setFlags(Qt::ItemIsEditable | Qt::ItemIsUserCheckable);
				itm->setCheckState(0, Qt::Unchecked);
				itm->setText(0, entryList.at(i));
				if (!netUtils->getInterfaceWifiStatus(entryList.at(i)))//if (wlanDeviceName != entryList.at(i))
				{
					itm->setIcon(0, QIcon(":/theme2_none.png"));
					itm->setText(1, "ETHERNET");
				}
				else
				{
					itm->setIcon(0, QIcon(":/wifi_radar_22x22.png"));
					itm->setText(1, "WIFI");
				}
				entryCount ++;
			}
		}
	}

	treeWidget->setSortingEnabled(true);
	treeWidget->sortByColumn(0, Qt::AscendingOrder);
	if (labelsList.count() > 0)
	{
		for (int j = 0; j < labelsList.count(); ++j)
			treeWidget->resizeColumnToContents (j);
	}
	
	if ( entryCount == 0)
		pbOk->setDisabled(true);
}

void NetworkStopDialog::disconnectProfilesOrInterfaces()
{
	//QStringList list;
	//QString commandStr;
	//int i=0;
	
	QTreeWidgetItemIterator joe( treeWidget , QTreeWidgetItemIterator::Checked );
	while (*joe)
	{
		//list << (*joe)->text(0);//qDebug() <<  qPrintable((*joe)->text(0)) << endl;
		if (targetMode == 0)
			queueList << netUtils->netcfgBinary + " -d " + (*joe)->text(0);//queueList << "netcfg2 -d " + (*joe)->text(0);
		if (targetMode == 1)
			queueList << netUtils->ifconfigBinary + " " + (*joe)->text(0) + " down";//queueList << "ifconfig " + (*joe)->text(0) + " down";
		++joe;
	}
	
	jobCount = 0;
	if (queueList.size()>0)
	{
		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
		layout->setEnabled(false);
		launchJob(jobCount);
	}
	
	/*
 	for (i = 0; i < list.count(); ++i)
 	{
		if (targetMode == 0)
			commandStr = "netcfg -d \"" + list.at(i) + "\"";
		if (targetMode == 1)
			commandStr = "ifconfig " + list.at(i) + " down";
		
		qDebug() <<  qPrintable(commandStr) << endl;
		//stopProcess->start(commandStr);
		//if (!stopProcess.waitForStarted())
			//	return;
			//if (!stopProcess.waitForFinished())
			//	return;

 	}
	*/
}

void NetworkStopDialog::launchJob(int idx)
{
	if (idx >= queueList.size())
		return;
	
	stopProcess->start(queueList.at(idx));
	if (!stopProcess->waitForStarted())
	{
		QMessageBox::warning(
				this,
				tr("Arch Assistant"),
				tr("Error : process could not be started."),
		  		QMessageBox::Ok
				);
	}
}
	
void NetworkStopDialog::jobEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	switch (exitStatus)
	{
		case QProcess::NormalExit:
			if (exitCode == 0)
			{
				qDebug() << "process is a success ! ExitCode = " << qPrintable(QString::number(exitCode));
			}
			else
			{
				errorsCount++;
				QMessageBox::warning(
						this,
		  				tr("Arch Assistant"),
			 			tr("Error : process ended with errors."),
						QMessageBox::Ok
						);
				qDebug() << "process returned an error ! ExitCode = " << qPrintable(QString::number(exitCode));
			}
			break;
		case QProcess::CrashExit:
			QMessageBox::warning(
						this,
						tr("Arch Assistant"),
						tr("Error : process crashed."),
						QMessageBox::Ok
						);
			qDebug() << "process crashed ! ";
			break;
		default:
			break;
	}
	
	//launch next job in queue if available
	jobCount ++;
	if (jobCount < queueList.size())
		launchJob(jobCount);
	else
	{
		QApplication::restoreOverrideCursor();
		layout->setEnabled(true);
		accept();
	}
}

// NetworkStopDialog::~NetworkStopDialog()
// {
// }
