#include "pacmanassistant.h"
#include <QtGui>
#include <QProcess>
#include <QIODevice>
#include <QTextEdit>
#include <iostream>
using namespace std;
//
//PacmanAssistant::PacmanAssistant(const QString &command, QStringList pacActions[9]) : QDialog()
PacmanAssistant::PacmanAssistant(int pacActionIndex) : QDialog()
{
	pacmanBin = "pacman";
	pacActions[0] << "-T" << trUtf8("(-Syu) Pacman Full Upgrade") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -Syu";
	pacActions[1] << "-T" << trUtf8("(-Su)   Pacman Simple Upgrade") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -Su";
	pacActions[2] << "-T" << trUtf8("(-Sy)   Pacman Synchronize") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -Sy";
	pacActions[3] << "-T" << trUtf8("(-Scc) Pacman Empty Cache") << "--noclose" << "-e" << pacmanBin << "-Scc";
	pacActions[4] << "request" << "-T" << trUtf8("(-S)     Pacman Install package(s)") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -S";
	pacActions[5] << "request" << "-T" << trUtf8("(-R)     Pacman Remove package(s)") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -R";
	pacActions[6] << "request" << "-T" << trUtf8("(-Ss)   Pacman Search package") << "--noclose" << "-e" << pacmanBin << "-Ss";
	pacActions[7] << "request" << "-T" << trUtf8("(-Si)    Pacman Get Info for package") << "--noclose" << "-e" << pacmanBin << "-Si";
	pacActions[8] << "request" << "-T" << trUtf8("(-Ql)    Pacman Show Files in package") << "--noclose" << "-e" << pacmanBin << "-Ql";
	
	// Setup GUI
	textEdit = new QTextEdit();
	
	//QLabel *pacCommandInfo = new QLabel(trUtf8("<b>%1</b>").arg(command));
	pacStateInfo = new QLabel(trUtf8("<i>pacman working, please wait ...</i>"));
	pbClose = new QPushButton(trUtf8("Abort"));
	pbStart = new QPushButton(trUtf8("Start"));
	pbStart->setDisabled(true);
	pbStart->setIcon(QIcon(":/run.png"));
	pbClose->setIcon(QIcon(":/fileclose.png"));
	leExtraArgs = new QLineEdit();
	cbPacmanActions = new QComboBox;
	
	// Fill combo box
	for (int i = 0; i < 9; ++i)
		{
			if (pacActions[i].at(0) == "request")
				cbPacmanActions->addItem(pacActions[i].at(2), pacActions[i].at(6));
			else
				cbPacmanActions->addItem(pacActions[i].at(1), pacActions[i].at(5));

			//if (i<6)
				//cbPacmanActions->item(i)->setDisabled(true);
		}
	
	cbPacmanActions->setCurrentIndex(pacActionIndex);
	//QPushButton *pbSave = new QPushButton("&Save", textEditDlg);
	
	QHBoxLayout *layoutH1 = new QHBoxLayout;
	layoutH1->addWidget( cbPacmanActions );
	layoutH1->addStretch();
	layoutH1->addWidget( leExtraArgs );
	layoutH1->addStretch();
	layoutH1->addWidget( pbStart );
	
	QHBoxLayout *layoutH2 = new QHBoxLayout;
	layoutH2->addWidget( pacStateInfo );
	layoutH2->addStretch();
	layoutH2->addWidget( pbClose );
	
	QVBoxLayout *layout = new QVBoxLayout;
	//layout->addWidget( pacCommandInfo );
	layout->addLayout( layoutH1 );
	layout->addWidget( textEdit );
	layout->addLayout( layoutH2 );
		
	//textEdit->setPlainText(systemInfoProcess->readAll());
	textEdit->setReadOnly(true);
		
	connect( pbClose, SIGNAL( clicked() ), this, SLOT( close() ) );
	connect( pbStart, SIGNAL( clicked() ), this, SLOT( pacProcessStart() ) );
	//connect( cbPacmanActions, SIGNAL( currentIndexChanged (int) ), this, SLOT( joe() ) );
	connect( cbPacmanActions, SIGNAL( currentIndexChanged (int) ), this, SLOT( cbPacmanActionsChanged() ) );
	
	
	setLayout(layout);
	//setModal(true);
	setSizeGripEnabled(true);
	setWindowTitle ( tr("Arch Assistant - Pacman") );
	setWindowIcon(QIcon(":/archlogo.png"));
	setMinimumSize(600, 400);
	
	//initDialog(command);
	//initDialog(pacActionIndex);
	
	// setup pacman process
	pacProcess = new QProcess(this);
	pacProcess->setProcessChannelMode(QProcess::MergedChannels);
	
	connect(pacProcess, SIGNAL(error(QProcess::ProcessError)),this, SLOT(pacProcessError(QProcess::ProcessError)));
	connect(pacProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(pacProcessEnded(int, QProcess::ExitStatus)));
	connect(pacProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(pacProcessOutput()));
	
	//qDebug() << cbPacmanActions->itemData(pacActionIndex) << endl;
	
	// launch pacman process if applies
	if ( pacActions[pacActionIndex].at(0) != "request" )
	{
		//qDebug() << cbPacmanActions->itemData(pacActionIndex) << endl;
		leExtraArgs->setDisabled(true);
		
		pacProcess->start(
					QString("%1 %2")
						.arg(pacmanBin)
						.arg(cbPacmanActions->itemData(pacActionIndex).toString()).toUtf8(),
					QIODevice::ReadWrite
					);
		//pacProcess->start(command, QIODevice::ReadWrite);
	
		if (!pacProcess->waitForStarted(3000))
		{
			cout << "error launching pacman" << endl;
			close();
		}
	}
	else
	{
		pbStart->setDisabled(false);
		pbClose->setText(trUtf8("Close"));
		pacStateInfo->clear();
		leExtraArgs->setDisabled(false);
		leExtraArgs->setFocus();
	}
}

void PacmanAssistant::pacProcessStart()
{
	if (cbPacmanActions->currentIndex() < 6)
	{
		QMessageBox::warning(this, tr("Pacman Assistant Error"),
							 "sorry, this feature has been disabled",
									 QMessageBox::Ok );
		return;
	}
	// launch pacman process
	if ( pacActions[cbPacmanActions->currentIndex()].at(0) != "request" )
	{
		//qDebug() << cbPacmanActions->itemData(pacActionIndex) << endl;
		
		pacProcess->start(
					QString("%1 %2")
						.arg(pacmanBin)
						.arg(cbPacmanActions->itemData(cbPacmanActions->currentIndex()).toString()).toUtf8(),
					QIODevice::ReadWrite
					);
		//pacProcess->start(command, QIODevice::ReadWrite);
	

	}
	else
	{
		pacProcess->start(
					QString("%1 %2 %3")
						.arg(pacmanBin)
						.arg(cbPacmanActions->itemData(cbPacmanActions->currentIndex()).toString())
						.arg(leExtraArgs->text()).toUtf8(),
					QIODevice::ReadWrite
					);
		//pbStart->setDisabled(false);
		//pbClose->setText(trUtf8("Close"));
		//pacStateInfo->clear();
	}
	
	if (!pacProcess->waitForStarted(3000))
	{
		cout << "error launching pacman" << endl;
		close();
	}
	
	pbStart->setDisabled(true);
	pbClose->setText(trUtf8("Abort"));
	pacStateInfo->setText(trUtf8("<i>pacman working, please wait ...</i>"));
	
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	
}

/*
//void PacmanAssistant::initDialog(QString command)
void PacmanAssistant::initDialog(int pacActionIndex)
{
	textEdit = new QTextEdit();
	
	//QLabel *pacCommandInfo = new QLabel(trUtf8("<b>%1</b>").arg(command));
	pacStateInfo = new QLabel(trUtf8("<i>pacman working, please wait ...</i>"));
	pbClose = new QPushButton(trUtf8("Abort"));
	pbStart = new QPushButton(trUtf8("Start"));
	pbStart->setDisabled(true);
	leExtraArgs = new QLineEdit();
	cbPacmanActions = new QComboBox;

	
	for (int i = 0; i < 10; ++i)
		{
			if (pacActions[i].at(0) == "request")
				cbPacmanActions->addItem(pacActions[i].at(2), pacActions[i].at(6));
			else
				cbPacmanActions->addItem(pacActions[i].at(1), pacActions[i].at(5) );				
		}
	
	cbPacmanActions->setCurrentIndex(pacActionIndex);
	//QPushButton *pbSave = new QPushButton("&Save", textEditDlg);
	
	QHBoxLayout *layoutH1 = new QHBoxLayout;
	layoutH1->addWidget( cbPacmanActions );
	layoutH1->addStretch();
	layoutH1->addWidget( leExtraArgs );
	layoutH1->addStretch();
	layoutH1->addWidget( pbStart );
	
	QHBoxLayout *layoutH2 = new QHBoxLayout;
	layoutH2->addWidget( pacStateInfo );
	layoutH2->addStretch();
	layoutH2->addWidget( pbClose );
	
	QVBoxLayout *layout = new QVBoxLayout;
	//layout->addWidget( pacCommandInfo );
	layout->addLayout( layoutH1 );
	layout->addWidget( textEdit );
	layout->addLayout( layoutH2 );
		
	//textEdit->setPlainText(systemInfoProcess->readAll());
	textEdit->setReadOnly(true);
		
	connect( pbClose, SIGNAL( clicked() ), this, SLOT( close() ) );
	
	setLayout(layout);
	//setModal(true);
	setSizeGripEnabled(true);
	setWindowTitle ( tr("Arch Assistant - Pacman") );
	setMinimumSize(600, 400);
}
*/
void PacmanAssistant::pacProcessEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	cout << "[OK] pacman process ended succesfully." << endl;
	pacStateInfo->setText(trUtf8("[OK] pacman process ended succesfully."));
	pbClose->setText(trUtf8("Ok"));
	pbStart->setDisabled(false);
	textEdit->insertPlainText("-----------\nDONE\n-----------\n\n");
	textEdit->ensureCursorVisible();
	QApplication::restoreOverrideCursor();
}

void PacmanAssistant::pacProcessError(QProcess::ProcessError errorStatus)
{
	cout << "[FAIL] pacman process ended with errors. (errorStatus = " << errorStatus << ")" << endl;
	pacStateInfo->setText(trUtf8("[FAIL] pacman process ended with errors."));
	pbClose->setText(trUtf8("Close"));
	pbStart->setDisabled(false);
	textEdit->insertPlainText("-----------\nFAILED\n-----------\n\n");
	QApplication::restoreOverrideCursor();
}

void PacmanAssistant::pacProcessOutput()
{
	
	QString tmpLine;
	
	while(pacProcess->canReadLine()) //while(pacProcess->bytesAvailable() > 0 )
	{
		//QByteArray buffer(pacProcess->read(1)); // read one byte = one character
		QByteArray buffer(pacProcess->readLine());
		//tmpLine.append(QString::fromLocal8Bit(buffer));
		tmpLine.append(QString(buffer));
		//qDebug() << buffer;
		//qDebug() << tmpLine;
	
		// display line after line in textEdit
		if (tmpLine.endsWith("\n"))
		{
			textEdit->insertPlainText(tmpLine);
			tmpLine.clear();
			//qDebug() << "reset here" << endl;
		}
			
	}
	
	// displays what remains in the buffer that's not ended by a '\n'
	textEdit->insertPlainText(QString(tmpLine).toUtf8());
	
	// check for a pacman question
	if (tmpLine.contains("? ["))
		{
			QString yesChar;
			QString noChar;
			QString tmpYesNoChars;
			
			//tmpYesNoChars = tmpLine.section("", tmpLine.lastIndexOf("["), tmpLine.lastIndexOf("]"));
			tmpYesNoChars = tmpLine.mid(tmpLine.lastIndexOf("[")+1,3);
			//tmpYesNoChars = tmpLine.mid(tmpLine.lastIndexOf("]")-1);
			yesChar = QString("%1\n").arg(tmpYesNoChars.left(1));
			noChar = QString("%1\n").arg(tmpYesNoChars.right(1));
			cout << "pacman asking for a question" << endl;
			//qDebug() << "carrouf" << tmpYesNoChars << " " << yesChar << " " << noChar << endl;
			
			QMessageBox msgBox;
			msgBox.setWindowTitle(trUtf8("Arch Assistant - Pacman Question"));
			msgBox.setText(trUtf8("%1").arg(QString(tmpLine)));
			msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
			
			switch (msgBox.exec())
			{
				case QMessageBox::Yes:
					//pacProcess->write("o\n");
					pacProcess->write(yesChar.toUtf8());
					//textEdit->insertPlainText(QString("O\n"));
					textEdit->insertPlainText(yesChar);
					break;
				case QMessageBox::No:
					pacProcess->write(noChar.toUtf8());
					//pacProcess->write("n\n");
					//textEdit->insertPlainText(QString("n\n"));
					textEdit->insertPlainText(noChar);
					break;
				default:
					// should never be reached
					break;
			}
		}
}

void PacmanAssistant::cbPacmanActionsChanged()
{
	if ( pacActions[cbPacmanActions->currentIndex()].at(0) == "request" )
		leExtraArgs->setDisabled(false);
	else
		leExtraArgs->setDisabled(true);
}
//
